@@ -9,6 +9,7 @@ class scapula():
         self.mesh = o3d.io.read_triangle_mesh(file_name)
         self.mesh.compute_vertex_normals()
         self.pcd = o3d.io.read_point_cloud(file_name)
+        self.guide_mesh = 0
 
     def get_points(self, point_local):
         self.p1 = point_local[0]
@@ -195,10 +196,14 @@ class scapula():
 
         mesh = self.mesh; point_coordinate = (0, 0, 0)
 
-        mesh2 = copy.deepcopy(self.mesh)
-        mesh2 = o3d.t.geometry.TriangleMesh.from_legacy(self.mesh)
-        scene = o3d.t.geometry.RaycastingScene()
-        _ = scene.add_triangles(mesh2)
+        try:
+            mesh2 = copy.deepcopy(self.mesh)
+            mesh2 = o3d.t.geometry.TriangleMesh.from_legacy(self.mesh)
+            scene = o3d.t.geometry.RaycastingScene()
+            _ = scene.add_triangles(mesh2)
+        except:
+            print ('mesh2')
+            return
 
         # 1.设定步长，角度1是1°，角度2是18°
         theta1 = 5/5; theta2 = 360/20
@@ -262,4 +267,74 @@ class scapula():
                         
             if (dis_origin != 100) and (dis_origin > location[0]):
                 location[0] = dis_origin; location[1] = know; location[2] = [i, j]
-            self.location = location
\ No newline at end of file
+        self.location = location
+
+        R = mesh.get_rotation_matrix_from_xyz((0, (5/5*location[2][0]+10)*np.pi / 180, 0))
+        mesh_cylinderchange1 = copy.deepcopy(self.cylinder)
+        mesh_cylinderchange1.rotate(R, center=point_coordinate)
+        R = mesh.get_rotation_matrix_from_xyz((0, 0, (360/20)*location[2][1]*np.pi / 180))
+        mesh_cylinderchange = copy.deepcopy(mesh_cylinderchange1)
+        mesh_cylinderchange.rotate(R, center=point_coordinate)
+        self.cylinder = copy.deepcopy(mesh_cylinderchange)
+
+    def find_guide(self):
+        mesh1 = o3d.t.geometry.TriangleMesh.from_legacy(self.mesh)
+        scene = o3d.t.geometry.RaycastingScene()
+        scene.add_triangles(mesh1)
+        a=np.array([])
+        r_circle = self.r
+        r_circle /= 2 / 3
+
+        p = []
+        for i in range(180):
+            for j in range(180):
+                for k in range(15):
+                    p.append([i, j, k])
+        for z1 in tqdm(p):
+            i = z1[0]; j = z1[1]; k = z1[2]
+            x=(-r_circle / 2) + r_circle / 180 * i; y=(self.mesh.vertices[self.id[0]][1]) - r_circle / 180 * j; z = (-10) + 0.8 * k
+            query_point = o3d.core.Tensor([[x,y,z]],dtype=o3d.core.Dtype.Float32)
+            ans = scene.compute_closest_points(query_point)
+            points=ans['points'].numpy()
+            triangle=ans['primitive_ids'][0].item()
+            a=np.append(a,triangle)
+            a=a.astype(int)
+
+        mesh2 = copy.deepcopy(self.mesh)
+        mesh2.triangles = o3d.utility.Vector3iVector(
+        np.asarray(mesh2.triangles)[a])
+        mesh2.triangle_normals = o3d.utility.Vector3dVector(
+        np.asarray(mesh2.triangle_normals)[a])
+        mesh2.paint_uniform_color([0.1, 0.1, 0.7])
+
+        mesh2.compute_vertex_normals()
+        pcd1 = mesh2.sample_points_uniformly(number_of_points=10000)
+
+        xyz= np.asarray(pcd1.points)
+        p = []
+        z1 = []
+        for i in range(10000):
+            for j in range(5):
+                z1.append([i, j])
+        for z in tqdm(z1):
+            i = z[0]; j = z[1]
+            q = [xyz[i, 0], xyz[i, 1], xyz[i, 2] - j * 0.5]
+            p.append(q)
+        p = np.array(p)
+        pcd2 = o3d.geometry.PointCloud()
+        pcd2.points = o3d.utility.Vector3dVector(p)
+        self.guide_pcd = pcd2
+    
+    def pcd_2_mesh(self, pcd2):
+
+        '''radius = 0.01  # 搜索半径
+        max_nn = 30  # 邻域内用于估算法线的最大点数
+        pcd2.estimate_normals(search_param=o3d.geometry.KDTreeSearchParamHybrid(radius, max_nn))'''
+
+        mesh4 = o3d.geometry.TriangleMesh.create_from_point_cloud_alpha_shape(pcd2, alpha=2)
+        mesh4.compute_vertex_normals()
+        mesh4.paint_uniform_color([0, 0.8,0.8])
+        return mesh4
+    
+    def show(self, l):
+        o3d.visualization.draw_geometries(l)
\ No newline at end of file
